/*
    eject.c
    Copyright (c) 2002  Edgar Hucek <hostmaster@ed-soft.at>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <i2c-dev.h>
#include "xbox.h"

void help() {
	printf("CD-ROM eject tool for the XBOX\n");
	printf("(c) 2002 Edgar Hucek <hostmaster@ed-soft.at>\n");
	printf("To youse this tool you must install and enable \nthe i2c-interface for the XBOX\n\n");
	printf("Usage: eject [OPTIONS]\n");
	printf("	Without parameters, eject ejects the CD-ROM tray\n");
	printf(" -t     Closes the CD-ROM tray\n");
	printf(" -h     Print this little help message\n");
}
int main(int argc, char *argv[])
{
  int res,i2cbus,file;
  int e1, e2, e3;
  int command = 0;
  char filename1[20];
  char filename2[20];
  char filename3[20];
  char *filename;


  if(argc > 1){
	  if (strcmp(argv[1],"-t") == 0) {
		  command = SMC_SUBCMD_EJECT_LOAD;
	  } else {
			  help();
			  exit(0);
	  }
  } else {
		  command = SMC_SUBCMD_EJECT_EJECT;
  }
  
  i2cbus = 0;

  sprintf(filename1,"/dev/i2c-%d",i2cbus);
  sprintf(filename2,"/dev/i2c%d",i2cbus);
  sprintf(filename3,"/dev/i2c/%d",i2cbus);
  if ((file = open(filename1,O_RDWR)) < 0) {
    e1 = errno;
    if ((file = open(filename2,O_RDWR)) < 0) {
      e2 = errno;
      if ((file = open(filename3,O_RDWR)) < 0) {
        e3 = errno;
        if(e1 == ENOENT && e2 == ENOENT && e3 == ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s', `%s', or `%s': %s\n",
                     filename1,filename2,filename3,strerror(ENOENT));
        }
        if (e1 != ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s' : %s\n",
                     filename1,strerror(e1));
          if(e1 == EACCES)
            fprintf(stderr,"Run as root?\n");
        }
        if (e2 != ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s' : %s\n",
                     filename2,strerror(e2));
          if(e2 == EACCES)
            fprintf(stderr,"Run as root?\n");
        }
        if (e3 != ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s' : %s\n",
                     filename3,strerror(e3));
          if(e3 == EACCES)
            fprintf(stderr,"Run as root?\n");
        }
        exit(1);
      } else {
         filename = filename3;
      }
    } else {
       filename = filename2;
    }
  } else {
    filename = filename1;
  }
  
  /* use FORCE so that we can look at registers even when
     a driver is also running */
  if (ioctl(file,I2C_SLAVE_FORCE,PIC_ADDRESS) < 0) {
    fprintf(stderr,"Error: Could not set address to %d: %s\n",PIC_ADDRESS,strerror(errno));
    exit(1);
  }
  res = i2c_smbus_write_byte_data(file, SMC_CMD_RESET_ON_EJECT, SMC_SUBCMD_RESET_ON_EJECT_DISABLE);
  if (ioctl(file,I2C_SLAVE_FORCE,PIC_ADDRESS) < 0) {
    fprintf(stderr,"Error: Could not set address to %d: %s\n",PIC_ADDRESS,strerror(errno));
    exit(1);
  }
  res = i2c_smbus_write_byte_data(file, SMC_CMD_EJECT, command);
  exit(0);
}
